(* Non-tail recursive version of summing function; recursive call must
   complete before adding on current i value. *)
let rec sum_em_NT i stop =
  if i=stop then
    stop
  else
    let rest = sum_em_NT (i+1) stop in
    i+rest
;;

(* Tail recursive summing function, uses an extra parameter which
   increases at each step; no state needs to be saved so the existing
   activation record can be reused *)
let rec sum_em_TR i stop sum =
  if i=stop then
    stop+sum
  else
    sum_em_TR (i+1) stop (i+sum)
;;

(* Typical implementation of a summing
   function; main interface takes start
   and stop; internal recursive helper
   function tracks index i, has
   parameter for sum to allow it to be
   tail recursive *)
let sum_em start stop =
  let rec helper i sum =
    if i=stop then
      stop+sum
    else
      helper (i+1) (i+sum)
  in
  helper start 0
;;

(* Factorial implementation with
   internal tail-recursive helper
   function; avoids the need to pass
   extra params to main function. *)
let factorial n =
  let rec fact i prod =
    if i > n then
      prod
    else
      fact (i+1) (i*prod)
  in
  fact 1 1
;;

(* 3 implementations of list_min; only B is tail recursive; comments
   explain why *)

let list_min_A list =
  let rec helper curmin lst =
    if lst=[] then
      curmin
    else
      let head = List.hd lst in
      let tail = List.tl lst in
      let tmin = helper curmin tail in (* recurse *)
      if head<tmin then                (* then select *)
        head                           (* answer *)
      else
        tmin
  in
  helper (List.hd list) (List.tl list)
;;

let list_min_B list =
  let rec helper curmin lst =
    if lst=[] then
      curmin
    else
      let head = List.hd lst in
      let tail = List.tl lst in
      let newmin =
        if head<curmin then
          head
        else
          curmin
      in
      helper newmin tail
  in
  helper (List.hd list) (List.tl list)
;;

let rec list_min_C list =
  let head = List.hd list in
  let tail = List.tl list in
  if tail=[] then
    head
  else
    let tmin = list_min_C tail in
    if head<tmin then
      head
    else
      tmin
;;

